/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        11. November 2014
 * $Revision:    V1.00
 *  
 * Project:      Flash Programming Functions for GigaDevice GD32F1x0 Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */ 

#include "FlashOS.H"        // FlashOS Structures

typedef volatile unsigned char  vu8;
typedef volatile unsigned long  vu32;
typedef volatile unsigned short vu16;

#define M8(adr)  (*((vu8  *) (adr)))
#define M16(adr) (*((vu16 *) (adr)))
#define M32(adr) (*((vu32 *) (adr)))										  
 

// Peripheral Memory Map
#define IWDG_BASE       0x40003000
#define FMC_BASE        0x40022000

#define IWDG            ((IWDG_TypeDef *) IWDG_BASE)
#define FMC             ((FMC_TypeDef*) FMC_BASE)

// FMC BANK size
#define BANK1_SIZE      0x00080000             

// Independent WATCHDOG
typedef struct {
  vu32 CTLR;
  vu32 PSR;
  vu32 RLDR;
  vu32 STR;
} IWDG_TypeDef;

// FMC Registers
typedef struct {
  vu32 RESR;                                     // offset  0x000
  vu32 UKEYR;                                    // offset  0x004
  vu32 OBKEYR;                                   // offset  0x008
  vu32 CSR;                                      // offset  0x00C
  vu32 CMR;                                      // offset  0x010
  vu32 AR;                                       // offset  0x014
  vu32 RESERVED;
  vu32 OPTR;                                     // offset  0x01C
  vu32 WPR;                                      // offset  0x020

} FMC_TypeDef;


// FMC Keys
#define RDPT_KEY          0x5AA5
#define FMC_KEY1          0x45670123
#define FMC_KEY2          0xCDEF89AB

// FMC Control Register definitions
#define FMC_CMR_PG        0x00000001
#define FMC_CMR_PE        0x00000002
#define FMC_CMR_ME        0x00000004
#define FMC_CMR_OBPG      0x00000010
#define FMC_CMR_OBER      0x00000020
#define FMC_CMR_START     0x00000040
#define FMC_CMR_LK        0x00000080
#define FMC_CMR_OBWE      0x00000200

// FMC Status Register definitions
#define FMC_CSR_BUSY      0x00000001
#define FMC_CSR_PGEF      0x00000004
#define FMC_CSR_WPEF      0x00000010
#define FMC_CSR_ENDF      0x00000020

unsigned long base_adr;



#ifdef FMC_OB

int Init (unsigned long adr, unsigned long clk, unsigned long fnc)
 {
  FMC->RESR  = 0x00000000;	                   // Zero Wait State
   
  FMC->UKEYR = FMC_KEY1;	                   // Unlock FMC 
  FMC->UKEYR = FMC_KEY2;

  FMC->OBKEYR = FMC_KEY1;	                   // Unlock Option Bytes
  FMC->OBKEYR = FMC_KEY2;

// Test if IWDG is running (IWDG in HW mode)
  if ((FMC->OPTR & 0x04) == 0x00)              
   {
   
    IWDG->CTLR  = 0x5555;                      // Enable write access to IWDG_PR and IWDG_RLR     
    IWDG->PSR  = 0x06;                         // Set prescaler to 256  
    IWDG->RLDR = 4095;                         // Set reload value to 4095
   }

   return (0);
}
#endif

#ifdef FMC_PE

 int Init (unsigned long adr, unsigned long clk, unsigned long fnc)
 {

  base_adr = adr & ~(BANK1_SIZE - 1);           // Align to Size Boundary
  FMC->RESR  = 0x00000000;		                // Zero Wait State
  FMC->UKEYR  = FMC_KEY1;	                    // Unlock FMC 
  FMC->UKEYR  = FMC_KEY2;


// Test if IWDG is running (IWDG in HW mode)
  if ((FMC->OPTR & 0x04) == 0x00) 		   	    
  {		
    IWDG->CTLR  = 0x5555;                       // Enable write access to IWDG_PR and IWDG_RLR     
    IWDG->PSR  = 0x06;                          // Set prescaler to 256  
    IWDG->RLDR = 4095;                          // Set reload value to 4095
  }

  return (0);
}
 
#endif



#ifdef FMC_OB
int UnInit (unsigned long fnc)
 {
  FMC->CMR |=  FMC_CMR_LK;
  FMC->CMR &= ~FMC_CMR_OBWE;                      // Lock FMC & Option Bytes
  return (0);
 }
#endif

#ifdef FMC_PE
   int UnInit (unsigned long fnc) 
{
  FMC->CMR  |=  FMC_CMR_LK;	                      // Lock FMC
  return (0);
} 
#endif




#ifdef FMC_OB
int EraseChip (void)
 {
  FMC->CMR |=  FMC_CMR_OBER;                      // Option Byte Erase Enabled 
  FMC->CMR |=  FMC_CMR_START;                     // Start Erase

  while (FMC->CSR & FMC_CSR_BUSY) 
  {
    IWDG->CTLR = 0xAAAA;                          // Reload IWDG
  }

  FMC->CMR &= ~FMC_CMR_OBER;                      // Option Byte Erase Disabled 

// Unprotect FMC

  FMC->CMR |=  FMC_CMR_OBPG;                      // Option Byte Programming Enabled

  M16(0x1FFFF800) = RDPT_KEY;                     // Program Half Word: RDPRT Key
  while (FMC->CSR & FMC_CSR_BUSY) 
  {
    IWDG->CTLR = 0xAAAA;                          // Reload IWDG
  }

  FMC->CMR &= ~FMC_CMR_OBPG;                      // Option Byte Programming Disabled

 
  if (FMC->CSR & (FMC_CSR_PGEF | FMC_CSR_WPEF))   // Check for Errors
  {
    FMC->CSR |= FMC_CSR_PGEF | FMC_CSR_WPEF;
    return (1);                                   // Failed
  }												 

  return (0);                                     // Done
}
#endif

#ifdef	FMC_PE
  int EraseChip (void)
{
  FMC->CMR  |=  FMC_CMR_ME;                       // Mass Erase Enabled
  FMC->CMR  |=  FMC_CMR_START;                    // Start Erase

  while (FMC->CSR  & FMC_CSR_BUSY)
   {
    IWDG->CTLR = 0xAAAA;                          // Reload IWDG
   }

  FMC->CMR  &= ~FMC_CMR_ME;                       // Mass Erase Disabled

  return (0);                                     // Done
}

#endif

#ifdef FMC_OB

int EraseSector (unsigned long adr)
 {
  FMC->CMR |=  FMC_CMR_OBER;                       // Option Byte Erase Enabled 
  FMC->CMR |=  FMC_CMR_START;                      // Start Erase

  while (FMC->CSR & FMC_CSR_BUSY) 
  {
    IWDG->CTLR = 0xAAAA;                           // Reload IWDG
  }

  FMC->CMR &= ~FMC_CMR_OBER;                       // Option Byte Erase Disabled 

  return (0);                                      // Done
 }
 
#endif

#ifdef FMC_PE
    int EraseSector (unsigned long adr)
  {
    FMC->CMR  |=  FMC_CMR_PE;                      // Page Erase Enabled 
    FMC->AR   =  adr;                              // Page Address
    FMC->CMR  |=  FMC_CMR_START;                   // Start Erase

    while (FMC->CSR  & FMC_CSR_BUSY)
	 {
      IWDG->CTLR = 0xAAAA;                         // Reload IWDG
    }

    FMC->CMR  &= ~FMC_CMR_PE;                      // Page Erase Disabled 

    return (0);                                    // Done
 }
#endif



#ifdef FMC_OB
int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat)
 {
  return (1);                                      // Always Force Erase
}

#endif

#ifdef FMC_OB
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf)
 {

  sz = (sz +1) & ~1;                                // Adjust size for Half Words
  
  while (sz) 
  {

    FMC->CMR |=  FMC_CMR_OBPG;                      // Option Byte Programming Enabled

    M16(adr) = *((unsigned short *)buf);            // Program Half Word
    while (FMC->CSR & FMC_CSR_BUSY) {
      IWDG->CTLR = 0xAAAA;                          // Reload IWDG
    }

    FMC->CMR &= ~FMC_CMR_OBPG;                      // Options Byte Programming Disabled


    if (FMC->CSR & (FMC_CSR_PGEF | FMC_CSR_WPEF))   // Check for Errors
	{
      FMC->CSR |= FMC_CSR_PGEF | FMC_CSR_WPEF;
      return (1);                                   // Failed
    }

                                                    // Go to next Half Word
    adr += 2;
    buf += 2;
    sz  -= 2;
   
  }

  return (0);                                       // Done
}

#endif

#ifdef FMC_PE
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) 
{
    sz = (sz + 3) & ~3;                             // Adjust size for  Words
   
    while (sz) {

      FMC->CMR  |=  FMC_CMR_PG;                     // Programming Enabled

      M32(adr) = *((unsigned long *)buf);           // Program Word
      while (FMC->CSR  & FMC_CSR_BUSY);

      FMC->CMR  &= ~FMC_CMR_PG;                     // Programming Disabled

                                                    // Check for Errors
      if (FMC->CSR & (FMC_CSR_PGEF | FMC_CSR_WPEF))
    {
      FMC->CSR |= FMC_CSR_PGEF | FMC_CSR_WPEF;
      return (1);                                   // Failed
    }
                                                    // Go to next  Word
      adr += 4;
      buf += 4;
      sz  -= 4;
    }

  return (0);                                       // Done
}

#endif



